@@ -893,7 +893,7 @@ func funcCallStep(callScope *EvalScope, stack *evalStack, thread Thread) bool {
 		// pretend we are still inside the function we called
 		fakeFunctionEntryScope(retScope, fncall.fn, int64(regs.SP()), regs.SP()-uint64(bi.Arch.PtrSize()))
 		var flags localsFlags
-		flags |= localsNoDeclLineCheck // if the function we are calling is an autogenerated stub then declaration lines have no meaning
+		flags |= localsNoDeclLineCheck | localsFakeFunctionEntryScope // if the function we are calling is an autogenerated stub then declaration lines have no meaning
 		if !bi.regabi {
 			flags |= localsTrustArgOrder
 		}
