@@ -836,6 +836,7 @@ func getEvalExpressionTestCases() []varTest {
 		{"int8(i6)", false, "12", "12", "int8", nil},
 		{"string(byteslice[0])", false, `"t"`, `"t"`, "string", nil},
 		{"string(runeslice[0])", false, `"t"`, `"t"`, "string", nil},
+		{"[]uint8(messageVar)", false, `[]uint8 len: 5, cap: 5, [1,2,3,4,5]`, `[]uint8 len: 5, cap: 5, [...]`, "[]uint8", nil},
 
 		// misc
 		{"i1", true, "1", "1", "int", nil},
@@ -1003,14 +1004,7 @@ func TestEvalExpression(t *testing.T) {
 					}
 					switch e := tc.err.(type) {
 					case *altError:
-						ok := false
-						for _, tgtErr := range e.errs {
-							if tgtErr == err.Error() {
-								ok = true
-								break
-							}
-						}
-						if !ok {
+						if !slices.Contains(e.errs, err.Error()) {
 							t.Fatalf("Unexpected error. Expected %s got %s", tc.err.Error(), err.Error())
 						}
 					default:
@@ -1511,14 +1505,7 @@ func testCallFunctionIntl(t *testing.T, grp *proc.TargetGroup, p *proc.Target, t
 		}
 		switch e := tc.err.(type) {
 		case *altError:
-			ok := false
-			for _, tgtErr := range e.errs {
-				if tgtErr == err.Error() {
-					ok = true
-					break
-				}
-			}
-			if !ok {
+			if !slices.Contains(e.errs, err.Error()) {
 				t.Fatalf("Unexpected error. Expected %s got %s", tc.err.Error(), err.Error())
 			}
 		default:
@@ -1598,14 +1585,7 @@ func TestIssue1531(t *testing.T) {
 			for i := 0; i < len(mv.Children); i += 2 {
 				cv := &mv.Children[i]
 				s := constant.StringVal(cv.Value)
-				found := false
-				for j := range keys {
-					if keys[j] == s {
-						found = true
-						break
-					}
-				}
-				if !found {
+				if !slices.Contains(keys, s) {
 					t.Errorf("key %q not allowed", s)
 					return
 				}
@@ -1962,14 +1942,7 @@ func TestClassicMap(t *testing.T) {
 					}
 					switch e := tc.err.(type) {
 					case *altError:
-						ok := false
-						for _, tgtErr := range e.errs {
-							if tgtErr == err.Error() {
-								ok = true
-								break
-							}
-						}
-						if !ok {
+						if !slices.Contains(e.errs, err.Error()) {
 							t.Fatalf("Unexpected error. Expected %s got %s", tc.err.Error(), err.Error())
 						}
 					default:
@@ -1995,3 +1968,36 @@ func TestCallFunctionRegisterArg(t *testing.T) {
 		assertNoError(proc.EvalExpressionWithCalls(grp, p.SelectedGoroutine(), "value.Type()", pnormalLoadConfig, true), t, "EvalExpressionWithCalls")
 	})
 }
+
+func TestCapturedVarVisibleOnFirstLine(t *testing.T) {
+	// Checks that a variable captured by a closure is visible on the first
+	// line of the closure function.
+	// See issue #4000
+	if !goversion.VersionAfterOrEqual(runtime.Version(), 1, 23) {
+		t.Skip("not implemented")
+	}
+	skipOn(t, "broken", "linux", "386")
+	withTestProcess("issue4000", t, func(p *proc.Target, grp *proc.TargetGroup, fixture protest.Fixture) {
+		addrs, err := proc.FindFileLocation(p, fixture.Source, 7)
+		assertNoError(err, t, "FindFileLocation")
+		found := false
+		for _, addr := range addrs {
+			_, _, fn := p.BinInfo().PCToLine(addr)
+			if fn != nil && strings.HasPrefix(fn.Name, "main.main.") {
+				found = true
+				_, err := p.SetBreakpoint(int(addr), addr, proc.UserBreakpoint, nil)
+				assertNoError(err, t, "SetBreakpoint")
+			}
+		}
+		if !found {
+			t.Fatal("could not find main.main.func1 at :7")
+		}
+		assertNoError(grp.Continue(), t, "Continue()") // this stops inside main.main.func1
+		v := evalVariable(p, t, "test")
+		cv := api.ConvertVar(v)
+		t.Logf("test variable: %s", cv.SinglelineString())
+		if tgt, s := `"a string"`, cv.SinglelineString(); s != tgt {
+			t.Fatalf("test variable expected %q got %q", tgt, s)
+		}
+	})
+}
