@@ -3,7 +3,7 @@ package terminal
 //lint:file-ignore ST1005 errors here can be capitalized
 
 import (
-	"bufio"
+	"errors"
 	"fmt"
 	"io"
 	"net/rpc"
@@ -59,6 +59,9 @@ type Term struct {
 	stdout   *transcriptWriter
 	InitFile string
 	displays []displayEntry
+	oldPid   int
+
+	stackTraceColors api.StackTraceColors
 
 	historyFile *os.File
 
@@ -75,6 +78,8 @@ type Term struct {
 
 	quittingMutex sync.Mutex
 	quitting      bool
+
+	traceNonInteractive bool
 }
 
 type displayEntry struct {
@@ -99,47 +104,90 @@ func New(client service.Client, conf *config.Config) *Term {
 		prompt: "(dlv) ",
 		line:   liner.NewLiner(),
 		cmds:   cmds,
-		stdout: &transcriptWriter{w: os.Stdout},
+		stdout: &transcriptWriter{pw: &pagingWriter{w: os.Stdout}},
 	}
 	t.line.SetCtrlZStop(true)
 
 	if strings.ToLower(os.Getenv("TERM")) != "dumb" {
-		t.stdout.w = getColorableWriter()
+		t.stdout.pw = &pagingWriter{w: getColorableWriter()}
 		t.stdout.colorEscapes = make(map[colorize.Style]string)
 		t.stdout.colorEscapes[colorize.NormalStyle] = terminalResetEscapeCode
-		wd := func(s string, defaultCode int) string {
-			if s == "" {
-				return fmt.Sprintf(terminalHighlightEscapeCode, defaultCode)
-			}
-			return s
-		}
-		t.stdout.colorEscapes[colorize.KeywordStyle] = conf.SourceListKeywordColor
-		t.stdout.colorEscapes[colorize.StringStyle] = wd(conf.SourceListStringColor, ansiGreen)
-		t.stdout.colorEscapes[colorize.NumberStyle] = conf.SourceListNumberColor
-		t.stdout.colorEscapes[colorize.CommentStyle] = wd(conf.SourceListCommentColor, ansiBrMagenta)
-		t.stdout.colorEscapes[colorize.ArrowStyle] = wd(conf.SourceListArrowColor, ansiYellow)
-		switch x := conf.SourceListLineColor.(type) {
-		case string:
-			t.stdout.colorEscapes[colorize.LineNoStyle] = x
-		case int:
-			if (x > ansiWhite && x < ansiBrBlack) || x < ansiBlack || x > ansiBrWhite {
-				x = ansiBlue
-			}
-			t.stdout.colorEscapes[colorize.LineNoStyle] = fmt.Sprintf(terminalHighlightEscapeCode, x)
-		case nil:
-			t.stdout.colorEscapes[colorize.LineNoStyle] = fmt.Sprintf(terminalHighlightEscapeCode, ansiBlue)
-		}
 	}
 
+	t.updateConfig()
+
 	if client != nil {
 		lcfg := t.loadConfig()
 		client.SetReturnValuesLoadConfig(&lcfg)
+		if state, err := client.GetState(); err == nil {
+			t.oldPid = state.Pid
+		}
 	}
 
 	t.starlarkEnv = starbind.New(starlarkContext{t}, t.stdout)
 	return t
 }
 
+func (t *Term) updateConfig() {
+	// These are always called together.
+	t.updateColorScheme()
+	t.updateTab()
+}
+
+func (t *Term) updateColorScheme() {
+	if t.stdout.colorEscapes == nil {
+		return
+	}
+
+	conf := t.conf
+	wd := func(s string, defaultCode int) string {
+		if s == "" {
+			return fmt.Sprintf(terminalHighlightEscapeCode, defaultCode)
+		}
+		return s
+	}
+	t.stdout.colorEscapes[colorize.KeywordStyle] = conf.SourceListKeywordColor
+	t.stdout.colorEscapes[colorize.StringStyle] = wd(conf.SourceListStringColor, ansiGreen)
+	t.stdout.colorEscapes[colorize.NumberStyle] = conf.SourceListNumberColor
+	t.stdout.colorEscapes[colorize.CommentStyle] = wd(conf.SourceListCommentColor, ansiBrMagenta)
+	t.stdout.colorEscapes[colorize.ArrowStyle] = wd(conf.SourceListArrowColor, ansiYellow)
+	t.stdout.colorEscapes[colorize.TabStyle] = wd(conf.SourceListTabColor, ansiBrBlack)
+	switch x := conf.SourceListLineColor.(type) {
+	case string:
+		t.stdout.colorEscapes[colorize.LineNoStyle] = x
+	case int:
+		if (x > ansiWhite && x < ansiBrBlack) || x < ansiBlack || x > ansiBrWhite {
+			x = ansiBlue
+		}
+		t.stdout.colorEscapes[colorize.LineNoStyle] = fmt.Sprintf(terminalHighlightEscapeCode, x)
+	case nil:
+		t.stdout.colorEscapes[colorize.LineNoStyle] = fmt.Sprintf(terminalHighlightEscapeCode, ansiBlue)
+	}
+
+	wd2 := func(s, defaultStr string) string {
+		if s == "" {
+			return defaultStr
+		}
+		return s
+	}
+
+	t.stackTraceColors.FunctionColor = wd2(conf.StackTraceFunctionColor, "\033[1m")
+	t.stackTraceColors.BasenameColor = wd2(conf.StackTraceBasenameColor, "\033[1m")
+	t.stackTraceColors.NormalColor = terminalResetEscapeCode
+}
+
+func (t *Term) updateTab() {
+	t.stdout.altTabString = t.conf.Tab
+}
+
+func (t *Term) SetTraceNonInteractive() {
+	t.traceNonInteractive = true
+}
+
+func (t *Term) IsTraceNonInteractive() bool {
+	return t.traceNonInteractive
+}
+
 // Close returns the terminal to its previous mode.
 func (t *Term) Close() {
 	t.line.Close()
@@ -195,7 +243,6 @@ func (t *Term) sigintGuard(ch <-chan os.Signal, multiClient bool) {
 			default:
 				fmt.Fprintln(t.stdout, "only p or q allowed")
 			}
-
 		} else {
 			fmt.Fprintf(t.stdout, "received SIGINT, stopping process (will not forward signal)\n")
 			_, err := t.client.Halt()
@@ -219,7 +266,7 @@ func (t *Term) Run() (int, error) {
 
 	fns := trie.New()
 	cmds := trie.New()
-	funcs, _ := t.client.ListFunctions("")
+	funcs, _ := t.client.ListFunctions("", 0)
 	for _, fn := range funcs {
 		fns.Add(fn, nil)
 	}
@@ -283,7 +330,7 @@ func (t *Term) Run() (int, error) {
 		fmt.Printf("Unable to open history file: %v. History will not be saved for this session.", err)
 	}
 	if _, err := t.line.ReadHistory(t.historyFile); err != nil {
-		fmt.Printf("Unable to read history file: %v", err)
+		fmt.Printf("Unable to read history file %s: %v\n", fullHistoryFile, err)
 	}
 
 	fmt.Println("Type 'help' for list of commands.")
@@ -313,7 +360,7 @@ func (t *Term) Run() (int, error) {
 				fmt.Fprintln(t.stdout, "exit")
 				return t.handleExit()
 			}
-			return 1, fmt.Errorf("Prompt for input failed.\n")
+			return 1, errors.New("Prompt for input failed.\n")
 		}
 		t.stdout.Echo(t.prompt + cmdstr + "\n")
 
@@ -344,6 +391,7 @@ func (t *Term) Run() (int, error) {
 		}
 
 		t.stdout.Flush()
+		t.stdout.pw.Reset()
 	}
 }
 
@@ -388,6 +436,10 @@ func (t *Term) formatPath(path string) string {
 }
 
 func (t *Term) promptForInput() (string, error) {
+	if t.stdout.colorEscapes != nil && t.conf.PromptColor != "" {
+		fmt.Fprint(os.Stdout, t.conf.PromptColor)
+		defer fmt.Fprint(os.Stdout, terminalResetEscapeCode)
+	}
 	l, err := t.line.Prompt(t.prompt)
 	if err != nil {
 		return "", err
@@ -401,13 +453,16 @@ func (t *Term) promptForInput() (string, error) {
 	return l, nil
 }
 
-func yesno(line *liner.State, question string) (bool, error) {
+var yesno = func(line *liner.State, question, defaultAnswer string) (bool, error) {
 	for {
 		answer, err := line.Prompt(question)
 		if err != nil {
 			return false, err
 		}
 		answer = strings.ToLower(strings.TrimSpace(answer))
+		if answer == "" {
+			answer = defaultAnswer
+		}
 		switch answer {
 		case "n", "no":
 			return false, nil
@@ -438,7 +493,7 @@ func (t *Term) handleExit() (int, error) {
 	if err != nil {
 		if isErrProcessExited(err) {
 			if t.client.IsMulticlient() {
-				answer, err := yesno(t.line, "Remote process has exited. Would you like to kill the headless instance? [Y/n] ")
+				answer, err := yesno(t.line, "Remote process has exited. Would you like to kill the headless instance? [Y/n] ", "yes")
 				if err != nil {
 					return 2, io.EOF
 				}
@@ -464,7 +519,7 @@ func (t *Term) handleExit() (int, error) {
 
 		doDetach := true
 		if t.client.IsMulticlient() {
-			answer, err := yesno(t.line, "Would you like to kill the headless instance? [Y/n] ")
+			answer, err := yesno(t.line, "Would you like to kill the headless instance? [Y/n] ", "yes")
 			if err != nil {
 				return 2, io.EOF
 			}
@@ -474,7 +529,7 @@ func (t *Term) handleExit() (int, error) {
 		if doDetach {
 			kill := true
 			if t.client.AttachedToExistingProcess() {
-				answer, err := yesno(t.line, "Would you like to kill the process? [Y/n] ")
+				answer, err := yesno(t.line, "Would you like to kill the process? [Y/n] ", "yes")
 				if err != nil {
 					return 2, io.EOF
 				}
@@ -570,7 +625,7 @@ func (t *Term) longCommandCanceled() bool {
 
 // RedirectTo redirects the output of this terminal to the specified writer.
 func (t *Term) RedirectTo(w io.Writer) {
-	t.stdout.w = w
+	t.stdout.pw.w = w
 }
 
 // isErrProcessExited returns true if `err` is an RPC error equivalent of proc.ErrProcessExited
@@ -578,80 +633,3 @@ func isErrProcessExited(err error) bool {
 	rpcError, ok := err.(rpc.ServerError)
 	return ok && strings.Contains(rpcError.Error(), "has exited with status")
 }
-
-// transcriptWriter writes to a io.Writer and also, optionally, to a
-// buffered file.
-type transcriptWriter struct {
-	fileOnly     bool
-	w            io.Writer
-	file         *bufio.Writer
-	fh           io.Closer
-	colorEscapes map[colorize.Style]string
-}
-
-func (w *transcriptWriter) Write(p []byte) (nn int, err error) {
-	if !w.fileOnly {
-		nn, err = w.w.Write(p)
-	}
-	if err == nil {
-		if w.file != nil {
-			return w.file.Write(p)
-		}
-	}
-	return
-}
-
-// ColorizePrint prints to out a syntax highlighted version of the text read from
-// reader, between lines startLine and endLine.
-func (w *transcriptWriter) ColorizePrint(path string, reader io.ReadSeeker, startLine, endLine, arrowLine int) error {
-	var err error
-	if !w.fileOnly {
-		err = colorize.Print(w.w, path, reader, startLine, endLine, arrowLine, w.colorEscapes)
-	}
-	if err == nil {
-		if w.file != nil {
-			reader.Seek(0, io.SeekStart)
-			return colorize.Print(w.file, path, reader, startLine, endLine, arrowLine, nil)
-		}
-	}
-	return err
-}
-
-// Echo outputs str only to the optional transcript file.
-func (w *transcriptWriter) Echo(str string) {
-	if w.file != nil {
-		w.file.WriteString(str)
-	}
-}
-
-// Flush flushes the optional transcript file.
-func (w *transcriptWriter) Flush() {
-	if w.file != nil {
-		w.file.Flush()
-	}
-}
-
-// CloseTranscript closes the optional transcript file.
-func (w *transcriptWriter) CloseTranscript() error {
-	if w.file == nil {
-		return nil
-	}
-	w.file.Flush()
-	w.fileOnly = false
-	err := w.fh.Close()
-	w.file = nil
-	w.fh = nil
-	return err
-}
-
-// TranscribeTo starts transcribing the output to the specified file. If
-// fileOnly is true the output will only go to the file, output to the
-// io.Writer will be suppressed.
-func (w *transcriptWriter) TranscribeTo(fh io.WriteCloser, fileOnly bool) {
-	if w.file == nil {
-		w.CloseTranscript()
-	}
-	w.fh = fh
-	w.file = bufio.NewWriter(fh)
-	w.fileOnly = fileOnly
-}
