@@ -1,11 +1,8 @@
 package api
 
 import (
-	"bytes"
 	"fmt"
 	"go/constant"
-	"go/printer"
-	"go/token"
 	"reflect"
 	"sort"
 	"strconv"
@@ -19,42 +16,44 @@ import (
 // ConvertLogicalBreakpoint converts a proc.LogicalBreakpoint into an API breakpoint.
 func ConvertLogicalBreakpoint(lbp *proc.LogicalBreakpoint) *Breakpoint {
 	b := &Breakpoint{
-		ID:            lbp.LogicalID,
-		FunctionName:  lbp.FunctionName,
-		File:          lbp.File,
-		Line:          lbp.Line,
-		Name:          lbp.Name,
-		Tracepoint:    lbp.Tracepoint,
-		TraceReturn:   lbp.TraceReturn,
-		Stacktrace:    lbp.Stacktrace,
-		Goroutine:     lbp.Goroutine,
-		Variables:     lbp.Variables,
-		LoadArgs:      LoadConfigFromProc(lbp.LoadArgs),
-		LoadLocals:    LoadConfigFromProc(lbp.LoadLocals),
-		TotalHitCount: lbp.TotalHitCount,
-		Disabled:      !lbp.Enabled,
-		UserData:      lbp.UserData,
+		ID:               lbp.LogicalID,
+		FunctionName:     lbp.FunctionName,
+		File:             lbp.File,
+		Line:             lbp.Line,
+		Name:             lbp.Name,
+		Tracepoint:       lbp.Tracepoint,
+		TraceReturn:      lbp.TraceReturn,
+		Stacktrace:       lbp.Stacktrace,
+		Goroutine:        lbp.Goroutine,
+		Variables:        lbp.Variables,
+		LoadArgs:         LoadConfigFromProc(lbp.LoadArgs),
+		LoadLocals:       LoadConfigFromProc(lbp.LoadLocals),
+		TotalHitCount:    lbp.TotalHitCount,
+		Disabled:         !lbp.Enabled(),
+		UserData:         lbp.UserData,
+		RootFuncName:     lbp.RootFuncName,
+		TraceFollowCalls: lbp.TraceFollowCalls,
 	}
 
 	b.HitCount = map[string]uint64{}
 	for idx := range lbp.HitCount {
-		b.HitCount[strconv.Itoa(idx)] = lbp.HitCount[idx]
+		b.HitCount[strconv.FormatInt(idx, 10)] = lbp.HitCount[idx]
 	}
 
-	if lbp.HitCond != nil {
-		b.HitCond = fmt.Sprintf("%s %d", lbp.HitCond.Op.String(), lbp.HitCond.Val)
-	}
+	b.HitCond = lbp.HitCond()
+	b.HitCondPerG = lbp.HitCondPerG
 
-	var buf bytes.Buffer
-	printer.Fprint(&buf, token.NewFileSet(), lbp.Cond)
-	b.Cond = buf.String()
+	b.Cond = lbp.Cond()
 
 	return b
 }
 
-// ConvertPhysicalBreakpoints adds informations from physical breakpoints to an API breakpoint.
-func ConvertPhysicalBreakpoints(b *Breakpoint, bps []*proc.Breakpoint) {
+// ConvertPhysicalBreakpoints adds information from physical breakpoints to an API breakpoint.
+func ConvertPhysicalBreakpoints(b *Breakpoint, lbp *proc.LogicalBreakpoint, pids []int, bps []*proc.Breakpoint) {
 	if len(bps) == 0 {
+		if lbp != nil {
+			b.ExprString = lbp.Set.ExprString
+		}
 		return
 	}
 
@@ -62,8 +61,9 @@ func ConvertPhysicalBreakpoints(b *Breakpoint, bps []*proc.Breakpoint) {
 	b.WatchType = WatchType(bps[0].WatchType)
 
 	lg := false
-	for _, bp := range bps {
+	for i, bp := range bps {
 		b.Addrs = append(b.Addrs, bp.Addr)
+		b.AddrPid = append(b.AddrPid, pids[i])
 		if b.FunctionName != bp.FunctionName && b.FunctionName != "" {
 			if !lg {
 				b.FunctionName = removeTypeParams(b.FunctionName)
@@ -94,10 +94,10 @@ func ConvertThread(th proc.Thread, bp *Breakpoint) *Thread {
 		file     string
 		line     int
 		pc       uint64
-		gid      int
+		gid      int64
 	)
 
-	loc, err := th.Location()
+	loc, err := proc.ThreadLocation(th)
 	if err == nil {
 		pc = loc.PC
 		file = loc.File
@@ -422,15 +422,21 @@ func ConvertRegisters(in *op.DwarfRegisters, dwarfRegisterToString func(int, *op
 			return a.DwarfNumber < b.DwarfNumber
 		}
 		return an < bn
-
 	})
 	return
 }
 
+// ConvertImage converts proc.Image to api.Image.
 func ConvertImage(image *proc.Image) Image {
-	return Image{Path: image.Path, Address: image.StaticBase}
+	err := image.LoadError()
+	lerr := ""
+	if err != nil {
+		lerr = err.Error()
+	}
+	return Image{Path: image.Path, Address: image.StaticBase, LoadError: lerr}
 }
 
+// ConvertDumpState converts proc.DumpState to api.DumpState.
 func ConvertDumpState(dumpState *proc.DumpState) *DumpState {
 	dumpState.Mutex.Lock()
 	defer dumpState.Mutex.Unlock()
@@ -447,3 +453,12 @@ func ConvertDumpState(dumpState *proc.DumpState) *DumpState {
 	}
 	return r
 }
+
+// ConvertTarget converts a proc.Target into a api.Target.
+func ConvertTarget(tgt *proc.Target, convertThreadBreakpoint func(proc.Thread) *Breakpoint) *Target {
+	return &Target{
+		Pid:           tgt.Pid(),
+		CmdLine:       tgt.CmdLine,
+		CurrentThread: ConvertThread(tgt.CurrentThread(), convertThreadBreakpoint(tgt.CurrentThread())),
+	}
+}
