@@ -3,6 +3,7 @@ package rpc2
 import (
 	"errors"
 	"fmt"
+	"regexp"
 	"sort"
 	"time"
 
@@ -126,7 +127,7 @@ type CommandOut struct {
 
 // Command interrupts, continues and steps through the program.
 func (s *RPCServer) Command(command api.DebuggerCommand, cb service.RPCCallback) {
-	st, err := s.debugger.Command(&command, cb.SetupDoneChan())
+	st, err := s.debugger.Command(&command, cb.SetupDoneChan(), cb.DisconnectChan())
 	if err != nil {
 		cb.Return(nil, err)
 		return
@@ -176,7 +177,7 @@ func (s *RPCServer) GetBreakpoint(arg GetBreakpointIn, out *GetBreakpointOut) er
 }
 
 type StacktraceIn struct {
-	Id     int
+	Id     int64
 	Depth  int
 	Full   bool
 	Defers bool // read deferred functions (equivalent to passing StacktraceReadDefers in Opts)
@@ -200,7 +201,6 @@ func (s *RPCServer) Stacktrace(arg StacktraceIn, out *StacktraceOut) error {
 	if arg.Defers {
 		arg.Opts |= api.StacktraceReadDefers
 	}
-	var err error
 	rawlocs, err := s.debugger.Stacktrace(arg.Id, arg.Depth, arg.Opts)
 	if err != nil {
 		return err
@@ -210,7 +210,7 @@ func (s *RPCServer) Stacktrace(arg StacktraceIn, out *StacktraceOut) error {
 }
 
 type AncestorsIn struct {
-	GoroutineID  int
+	GoroutineID  int64
 	NumAncestors int
 	Depth        int
 }
@@ -242,6 +242,10 @@ func (s *RPCServer) ListBreakpoints(arg ListBreakpointsIn, out *ListBreakpointsO
 
 type CreateBreakpointIn struct {
 	Breakpoint api.Breakpoint
+
+	LocExpr             string
+	SubstitutePathRules [][2]string
+	Suspended           bool
 }
 
 type CreateBreakpointOut struct {
@@ -249,14 +253,14 @@ type CreateBreakpointOut struct {
 }
 
 // CreateBreakpoint creates a new breakpoint. The client is expected to populate `CreateBreakpointIn`
-// with an `api.Breakpoint` struct describing where to set the breakpoing. For more information on
+// with an `api.Breakpoint` struct describing where to set the breakpoint. For more information on
 // how to properly request a breakpoint via the `api.Breakpoint` struct see the documentation for
 // `debugger.CreateBreakpoint` here: https://pkg.go.dev/github.com/go-delve/delve/service/debugger#Debugger.CreateBreakpoint.
 func (s *RPCServer) CreateBreakpoint(arg CreateBreakpointIn, out *CreateBreakpointOut) error {
 	if err := api.ValidBreakpointName(arg.Breakpoint.Name); err != nil {
 		return err
 	}
-	createdbp, err := s.debugger.CreateBreakpoint(&arg.Breakpoint)
+	createdbp, err := s.debugger.CreateBreakpoint(&arg.Breakpoint, arg.LocExpr, arg.SubstitutePathRules, arg.Suspended)
 	if err != nil {
 		return err
 	}
@@ -385,8 +389,8 @@ func (s *RPCServer) ListThreads(arg ListThreadsIn, out *ListThreadsOut) (err err
 	if err != nil {
 		return err
 	}
-	s.debugger.LockTarget()
-	defer s.debugger.UnlockTarget()
+	_, unlock := s.debugger.LockTargetGroup()
+	defer unlock()
 	out.Threads = api.ConvertThreads(threads, s.debugger.ConvertThreadBreakpoint)
 	return nil
 }
@@ -408,8 +412,8 @@ func (s *RPCServer) GetThread(arg GetThreadIn, out *GetThreadOut) error {
 	if t == nil {
 		return fmt.Errorf("no thread with id %d", arg.Id)
 	}
-	s.debugger.LockTarget()
-	defer s.debugger.UnlockTarget()
+	_, unlock := s.debugger.LockTargetGroup()
+	defer unlock()
 	out.Thread = api.ConvertThread(t, s.debugger.ConvertThreadBreakpoint(t))
 	return nil
 }
@@ -457,17 +461,18 @@ func (s *RPCServer) ListRegisters(arg ListRegistersIn, out *ListRegistersOut) er
 	}
 
 	var regs *op.DwarfRegisters
+	var dwarfRegisterToString proc.DwarfRegisterToStringFunc
 	var err error
 
 	if arg.Scope != nil {
-		regs, err = s.debugger.ScopeRegisters(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.IncludeFp)
+		regs, dwarfRegisterToString, err = s.debugger.ScopeRegisters(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall)
 	} else {
-		regs, err = s.debugger.ThreadRegisters(arg.ThreadID, arg.IncludeFp)
+		regs, dwarfRegisterToString, err = s.debugger.ThreadRegisters(arg.ThreadID)
 	}
 	if err != nil {
 		return err
 	}
-	out.Regs = api.ConvertRegisters(regs, s.debugger.DwarfRegisterToString, arg.IncludeFp)
+	out.Regs = api.ConvertRegisters(regs, dwarfRegisterToString, arg.IncludeFp)
 	out.Registers = out.Regs.String()
 
 	return nil
@@ -530,7 +535,8 @@ func (s *RPCServer) Eval(arg EvalIn, out *EvalOut) error {
 	if cfg == nil {
 		cfg = &api.LoadConfig{FollowPointers: true, MaxVariableRecurse: 1, MaxStringLen: 64, MaxArrayValues: 64, MaxStructFields: -1}
 	}
-	v, err := s.debugger.EvalVariableInScope(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Expr, *api.LoadConfigToProc(cfg))
+	pcfg := *api.LoadConfigToProc(cfg)
+	v, err := s.debugger.EvalVariableInScope(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Expr, pcfg)
 	if err != nil {
 		return err
 	}
@@ -572,7 +578,8 @@ func (s *RPCServer) ListSources(arg ListSourcesIn, out *ListSourcesOut) error {
 }
 
 type ListFunctionsIn struct {
-	Filter string
+	Filter      string
+	FollowCalls int
 }
 
 type ListFunctionsOut struct {
@@ -581,7 +588,7 @@ type ListFunctionsOut struct {
 
 // ListFunctions lists all functions in the process matching filter.
 func (s *RPCServer) ListFunctions(arg ListFunctionsIn, out *ListFunctionsOut) error {
-	fns, err := s.debugger.Functions(arg.Filter)
+	fns, err := s.debugger.Functions(arg.Filter, arg.FollowCalls)
 	if err != nil {
 		return err
 	}
@@ -613,6 +620,8 @@ type ListGoroutinesIn struct {
 
 	Filters []api.ListGoroutinesFilter
 	api.GoroutineGroupingOptions
+
+	EvalScope *api.EvalScope
 }
 
 type ListGoroutinesOut struct {
@@ -653,21 +662,51 @@ type ListGoroutinesOut struct {
 // be grouped with the specified criterion.
 // If the value of arg.GroupBy is GoroutineLabel goroutines will
 // be grouped by the value of the label with key GroupByKey.
-// For each group a maximum of MaxExamples example goroutines are
+// For each group a maximum of MaxGroupMembers example goroutines are
 // returned, as well as the total number of goroutines in the group.
 func (s *RPCServer) ListGoroutines(arg ListGoroutinesIn, out *ListGoroutinesOut) error {
 	//TODO(aarzilli): if arg contains a running goroutines filter (not negated)
 	// and start == 0 and count == 0 then we can optimize this by just looking
 	// at threads directly.
-	gs, nextg, err := s.debugger.Goroutines(arg.Start, arg.Count)
+
+	var gs []*proc.G
+	var nextg int
+	var err error
+	var gsLoaded bool
+
+	for _, filter := range arg.Filters {
+		if filter.Kind == api.GoroutineWaitingOnChannel {
+			if filter.Negated {
+				return errors.New("channel filter can not be negated")
+			}
+			if arg.Count == 0 {
+				return errors.New("count == 0 not allowed with a channel filter")
+			}
+			if arg.EvalScope == nil {
+				return errors.New("channel filter without eval scope")
+			}
+			gs, err = s.debugger.ChanGoroutines(arg.EvalScope.GoroutineID, arg.EvalScope.Frame, arg.EvalScope.DeferredCall, filter.Arg, arg.Start, arg.Count)
+			if len(gs) == arg.Count {
+				nextg = arg.Start + len(gs)
+			} else {
+				nextg = -1
+			}
+			gsLoaded = true
+			break
+		}
+	}
+
+	if !gsLoaded {
+		gs, nextg, err = s.debugger.Goroutines(arg.Start, arg.Count)
+	}
 	if err != nil {
 		return err
 	}
 	gs = s.debugger.FilterGoroutines(gs, arg.Filters)
 	gs, out.Groups, out.TooManyGroups = s.debugger.GroupGoroutines(gs, &arg.GoroutineGroupingOptions)
-	s.debugger.LockTarget()
-	defer s.debugger.UnlockTarget()
-	out.Goroutines = api.ConvertGoroutines(s.debugger.Target(), gs)
+	tgrp, unlock := s.debugger.LockTargetGroup()
+	defer unlock()
+	out.Goroutines = api.ConvertGoroutines(tgrp.Selected, gs)
 	out.Nextg = nextg
 	return nil
 }
@@ -680,8 +719,8 @@ type AttachedToExistingProcessOut struct {
 }
 
 // AttachedToExistingProcess returns whether we attached to a running process or not
-func (c *RPCServer) AttachedToExistingProcess(arg AttachedToExistingProcessIn, out *AttachedToExistingProcessOut) error {
-	if c.config.Debugger.AttachPid != 0 {
+func (s *RPCServer) AttachedToExistingProcess(arg AttachedToExistingProcessIn, out *AttachedToExistingProcessOut) error {
+	if s.config.Debugger.AttachPid != 0 {
 		out.Answer = true
 	}
 	return nil
@@ -701,7 +740,8 @@ type FindLocationIn struct {
 }
 
 type FindLocationOut struct {
-	Locations []api.Location
+	Locations         []api.Location
+	SubstituteLocExpr string // if this isn't an empty string it should be passed as the location expression for CreateBreakpoint instead of the original location expression
 }
 
 // FindLocation returns concrete location information described by a location expression.
@@ -717,9 +757,9 @@ type FindLocationOut struct {
 //	* *<address> returns the location corresponding to the specified address
 //
 // NOTE: this function does not actually set breakpoints.
-func (c *RPCServer) FindLocation(arg FindLocationIn, out *FindLocationOut) error {
+func (s *RPCServer) FindLocation(arg FindLocationIn, out *FindLocationOut) error {
 	var err error
-	out.Locations, err = c.debugger.FindLocation(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Loc, arg.IncludeNonExecutableLines, arg.SubstitutePathRules)
+	out.Locations, out.SubstituteLocExpr, err = s.debugger.FindLocation(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Loc, arg.IncludeNonExecutableLines, arg.SubstitutePathRules)
 	return err
 }
 
@@ -740,15 +780,14 @@ type DisassembleOut struct {
 // Scope is used to mark the instruction the specified goroutine is stopped at.
 //
 // Disassemble will also try to calculate the destination address of an absolute indirect CALL if it happens to be the instruction the selected goroutine is stopped at.
-func (c *RPCServer) Disassemble(arg DisassembleIn, out *DisassembleOut) error {
-	var err error
-	insts, err := c.debugger.Disassemble(arg.Scope.GoroutineID, arg.StartPC, arg.EndPC)
+func (s *RPCServer) Disassemble(arg DisassembleIn, out *DisassembleOut) error {
+	insts, err := s.debugger.Disassemble(arg.Scope.GoroutineID, arg.StartPC, arg.EndPC)
 	if err != nil {
 		return err
 	}
 	out.Disassemble = make(api.AsmInstructions, len(insts))
 	for i := range insts {
-		out.Disassemble[i] = api.ConvertAsmInstruction(insts[i], c.debugger.AsmInstructionText(&insts[i], proc.AssemblyFlavour(arg.Flavour)))
+		out.Disassemble[i] = api.ConvertAsmInstruction(insts[i], s.debugger.AsmInstructionText(&insts[i], proc.AssemblyFlavour(arg.Flavour)))
 	}
 	return nil
 }
@@ -788,7 +827,6 @@ type ListCheckpointsOut struct {
 }
 
 func (s *RPCServer) ListCheckpoints(arg ListCheckpointsIn, out *ListCheckpointsOut) error {
-	var err error
 	cps, err := s.debugger.Checkpoints()
 	if err != nil {
 		return err
@@ -874,12 +912,13 @@ func (s *RPCServer) ListDynamicLibraries(in ListDynamicLibrariesIn, out *ListDyn
 	return nil
 }
 
-// ListPackagesBuildInfoIn holds the arguments of ListPackages.
+// ListPackagesBuildInfoIn holds the arguments of ListPackagesBuildInfo.
 type ListPackagesBuildInfoIn struct {
 	IncludeFiles bool
+	Filter       string // if not empty, returns only packages matching the regexp.
 }
 
-// ListPackagesBuildInfoOut holds the return values of ListPackages.
+// ListPackagesBuildInfoOut holds the return values of ListPackagesBuildInfo.
 type ListPackagesBuildInfoOut struct {
 	List []api.PackageBuildInfo
 }
@@ -890,9 +929,20 @@ type ListPackagesBuildInfoOut struct {
 // Note that the directory path is a best guess and may be wrong is a tool
 // other than cmd/go is used to perform the build.
 func (s *RPCServer) ListPackagesBuildInfo(in ListPackagesBuildInfoIn, out *ListPackagesBuildInfoOut) error {
+	var pattern *regexp.Regexp
+	if in.Filter != "" {
+		p, err := regexp.Compile(in.Filter)
+		if err != nil {
+			return fmt.Errorf("invalid Filter pattern: %v", err)
+		}
+		pattern = p
+	}
 	pkgs := s.debugger.ListPackagesBuildInfo(in.IncludeFiles)
 	out.List = make([]api.PackageBuildInfo, 0, len(pkgs))
 	for _, pkg := range pkgs {
+		if pattern != nil && !pattern.MatchString(pkg.ImportPath) {
+			continue
+		}
 		var files []string
 
 		if len(pkg.Files) > 0 {
@@ -925,9 +975,11 @@ type ExaminedMemoryOut struct {
 	IsLittleEndian bool
 }
 
+const ExamineMemoryLengthLimit = 1 << 16
+
 func (s *RPCServer) ExamineMemory(arg ExamineMemoryIn, out *ExaminedMemoryOut) error {
-	if arg.Length > 1000 {
-		return fmt.Errorf("len must be less than or equal to 1000")
+	if arg.Length > ExamineMemoryLengthLimit {
+		return fmt.Errorf("len must be less than or equal to %d", ExamineMemoryLengthLimit)
 	}
 	Mem, err := s.debugger.ExamineMemory(arg.Address, arg.Length)
 	if err != nil {
@@ -1029,3 +1081,82 @@ func (s *RPCServer) BuildID(arg BuildIDIn, out *BuildIDOut) error {
 	out.BuildID = s.debugger.BuildID()
 	return nil
 }
+
+type ListTargetsIn struct {
+}
+
+type ListTargetsOut struct {
+	Targets []api.Target
+}
+
+// ListTargets returns the list of targets we are currently attached to.
+func (s *RPCServer) ListTargets(arg ListTargetsIn, out *ListTargetsOut) error {
+	tgrp, unlock := s.debugger.LockTargetGroup()
+	defer unlock()
+	out.Targets = []api.Target{}
+	for _, tgt := range tgrp.Targets() {
+		if _, err := tgt.Valid(); err == nil {
+			out.Targets = append(out.Targets, *api.ConvertTarget(tgt, s.debugger.ConvertThreadBreakpoint))
+		}
+	}
+	return nil
+}
+
+type FollowExecIn struct {
+	Enable bool
+	Regex  string
+}
+
+type FollowExecOut struct {
+}
+
+// FollowExec enables or disables follow exec mode.
+func (s *RPCServer) FollowExec(arg FollowExecIn, out *FollowExecOut) error {
+	return s.debugger.FollowExec(arg.Enable, arg.Regex)
+}
+
+type FollowExecEnabledIn struct {
+}
+
+type FollowExecEnabledOut struct {
+	Enabled bool
+}
+
+// FollowExecEnabled returns true if follow exec mode is enabled.
+func (s *RPCServer) FollowExecEnabled(arg FollowExecEnabledIn, out *FollowExecEnabledOut) error {
+	out.Enabled = s.debugger.FollowExecEnabled()
+	return nil
+}
+
+type DebugInfoDirectoriesIn struct {
+	Set  bool
+	List []string
+}
+
+type DebugInfoDirectoriesOut struct {
+	List []string
+}
+
+func (s *RPCServer) DebugInfoDirectories(arg DebugInfoDirectoriesIn, out *DebugInfoDirectoriesOut) error {
+	if arg.Set {
+		s.debugger.SetDebugInfoDirectories(arg.List)
+	}
+	out.List = s.debugger.DebugInfoDirectories()
+	return nil
+}
+
+type GuessSubstitutePathIn struct {
+	Args api.GuessSubstitutePathIn
+}
+
+type GuessSubstitutePathOut struct {
+	List [][2]string
+}
+
+func (s *RPCServer) GuessSubstitutePath(arg GuessSubstitutePathIn, out *GuessSubstitutePathOut) error {
+	m := s.debugger.GuessSubstitutePath(&arg.Args)
+	for k, v := range m {
+		out.List = append(out.List, [2]string{k, v})
+	}
+	return nil
+}
