@@ -1,8 +1,10 @@
 package rpccommon
 
 import (
+	"bufio"
 	"bytes"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net"
@@ -12,46 +14,44 @@ import (
 	"reflect"
 	"runtime"
 	"sync"
-	"unicode"
-	"unicode/utf8"
 
 	"github.com/go-delve/delve/pkg/logflags"
 	"github.com/go-delve/delve/pkg/version"
 	"github.com/go-delve/delve/service"
 	"github.com/go-delve/delve/service/api"
+	"github.com/go-delve/delve/service/dap"
 	"github.com/go-delve/delve/service/debugger"
 	"github.com/go-delve/delve/service/internal/sameuser"
-	"github.com/go-delve/delve/service/rpc1"
 	"github.com/go-delve/delve/service/rpc2"
-	"github.com/sirupsen/logrus"
 )
 
+//go:generate go run ../../_scripts/gen-suitablemethods.go suitablemethods
+
 // ServerImpl implements a JSON-RPC server that can switch between two
 // versions of the API.
 type ServerImpl struct {
 	// config is all the information necessary to start the debugger and server.
 	config *service.Config
-	// listener is used to serve HTTP.
+	// listener is used to serve JSON-RPC.
 	listener net.Listener
 	// stopChan is used to stop the listener goroutine.
 	stopChan chan struct{}
 	// debugger is the debugger service.
 	debugger *debugger.Debugger
-	// s1 is APIv1 server.
-	s1 *rpc1.RPCServer
 	// s2 is APIv2 server.
 	s2 *rpc2.RPCServer
 	// maps of served methods, one for each supported API.
 	methodMaps []map[string]*methodType
-	log        *logrus.Entry
+	log        logflags.Logger
 }
 
 type RPCCallback struct {
-	s         *ServerImpl
-	sending   *sync.Mutex
-	codec     rpc.ServerCodec
-	req       rpc.Request
-	setupDone chan struct{}
+	s              *ServerImpl
+	sending        *sync.Mutex
+	codec          rpc.ServerCodec
+	req            rpc.Request
+	setupDone      chan struct{}
+	disconnectChan chan struct{}
 }
 
 var _ service.RPCCallback = &RPCCallback{}
@@ -62,8 +62,7 @@ type RPCServer struct {
 }
 
 type methodType struct {
-	method      reflect.Method
-	Rcvr        reflect.Value
+	method      reflect.Value
 	ArgType     reflect.Type
 	ReplyType   reflect.Type
 	Synchronous bool
@@ -77,7 +76,7 @@ func NewServer(config *service.Config) *ServerImpl {
 	}
 	if config.Debugger.Foreground {
 		// Print listener address
-		logflags.WriteAPIListeningMessage(config.Listener.Addr().String())
+		logflags.WriteAPIListeningMessage(config.Listener.Addr())
 		logger.Debug("API server pid = ", os.Getpid())
 	}
 	return &ServerImpl{
@@ -90,25 +89,29 @@ func NewServer(config *service.Config) *ServerImpl {
 
 // Stop stops the JSON-RPC server.
 func (s *ServerImpl) Stop() error {
+	s.log.Debug("stopping")
 	close(s.stopChan)
 	if s.config.AcceptMulti {
 		s.listener.Close()
 	}
+	if s.debugger.IsRunning() {
+		s.debugger.Command(&api.DebuggerCommand{Name: api.Halt}, nil, nil)
+	}
 	kill := s.config.Debugger.AttachPid == 0
 	return s.debugger.Detach(kill)
 }
 
-// Run starts a debugger and exposes it with an HTTP server. The debugger
-// itself can be stopped with the `detach` API. Run blocks until the HTTP
-// server stops.
+// Run starts a debugger and exposes it with an JSON-RPC server. The debugger
+// itself can be stopped with the `detach` API.
 func (s *ServerImpl) Run() error {
 	var err error
 
-	if s.config.APIVersion < 2 {
-		s.config.APIVersion = 1
+	if s.config.APIVersion == 0 {
+		s.config.APIVersion = 2
 	}
-	if s.config.APIVersion > 2 {
-		return fmt.Errorf("unknown API version")
+
+	if s.config.APIVersion != 2 {
+		return errors.New("unknown API version")
 	}
 
 	// Create and start the debugger
@@ -117,19 +120,17 @@ func (s *ServerImpl) Run() error {
 		return err
 	}
 
-	s.s1 = rpc1.NewServer(s.config, s.debugger)
 	s.s2 = rpc2.NewServer(s.config, s.debugger)
 
 	rpcServer := &RPCServer{s}
 
 	s.methodMaps = make([]map[string]*methodType, 2)
 
-	s.methodMaps[0] = map[string]*methodType{}
 	s.methodMaps[1] = map[string]*methodType{}
-	suitableMethods(s.s1, s.methodMaps[0], s.log)
-	suitableMethods(rpcServer, s.methodMaps[0], s.log)
-	suitableMethods(s.s2, s.methodMaps[1], s.log)
-	suitableMethods(rpcServer, s.methodMaps[1], s.log)
+
+	suitableMethods2(s.s2, s.methodMaps[1])
+	suitableMethodsCommon(rpcServer, s.methodMaps[1])
+	finishMethodsMapInit(s.methodMaps[1])
 
 	go func() {
 		defer s.listener.Close()
@@ -152,7 +153,7 @@ func (s *ServerImpl) Run() error {
 				}
 			}
 
-			go s.serveJSONCodec(c)
+			go s.serveConnectionDemux(c)
 			if !s.config.AcceptMulti {
 				break
 			}
@@ -161,96 +162,44 @@ func (s *ServerImpl) Run() error {
 	return nil
 }
 
-// Precompute the reflect type for error.  Can't use error directly
-// because Typeof takes an empty interface value.  This is annoying.
-var typeOfError = reflect.TypeOf((*error)(nil)).Elem()
-
-// Is this an exported - upper case - name?
-func isExported(name string) bool {
-	ch, _ := utf8.DecodeRuneInString(name)
-	return unicode.IsUpper(ch)
+type bufReadWriteCloser struct {
+	*bufio.Reader
+	io.WriteCloser
 }
 
-// Is this type exported or a builtin?
-func isExportedOrBuiltinType(t reflect.Type) bool {
-	for t.Kind() == reflect.Ptr {
-		t = t.Elem()
-	}
-	// PkgPath will be non-empty even for an exported type,
-	// so we need to check the type name as well.
-	return isExported(t.Name()) || t.PkgPath() == ""
-}
-
-// Fills methods map with the methods of receiver that should be made
-// available through the RPC interface.
-// These are all the public methods of rcvr that have one of those
-// two signatures:
-//  func (rcvr ReceiverType) Method(in InputType, out *ReplyType) error
-//  func (rcvr ReceiverType) Method(in InputType, cb service.RPCCallback)
-func suitableMethods(rcvr interface{}, methods map[string]*methodType, log *logrus.Entry) {
-	typ := reflect.TypeOf(rcvr)
-	rcvrv := reflect.ValueOf(rcvr)
-	sname := reflect.Indirect(rcvrv).Type().Name()
-	if sname == "" {
-		log.Debugf("rpc.Register: no service name for type %s", typ)
+func (s *ServerImpl) serveConnectionDemux(c io.ReadWriteCloser) {
+	conn := &bufReadWriteCloser{bufio.NewReader(c), c}
+	b, err := conn.Peek(1)
+	if err != nil {
+		s.log.Warnf("error determining new connection protocol: %v", err)
 		return
 	}
-	for m := 0; m < typ.NumMethod(); m++ {
-		method := typ.Method(m)
-		mname := method.Name
-		mtype := method.Type
-		// method must be exported
-		if method.PkgPath != "" {
-			continue
-		}
-		// Method needs three ins: (receive, *args, *reply) or (receiver, *args, *RPCCallback)
-		if mtype.NumIn() != 3 {
-			log.Warn("method", mname, "has wrong number of ins:", mtype.NumIn())
-			continue
-		}
-		// First arg need not be a pointer.
-		argType := mtype.In(1)
-		if !isExportedOrBuiltinType(argType) {
-			log.Warn(mname, "argument type not exported:", argType)
-			continue
-		}
+	if b[0] == 'C' { // C is for DAP's Content-Length
+		s.log.Debugf("serving DAP on new connection")
+		ds := dap.NewSession(conn, &dap.Config{Config: s.config, StopTriggered: s.stopChan}, s.debugger)
+		go ds.ServeDAPCodec()
+	} else {
+		s.log.Debugf("serving JSON-RPC on new connection")
+		go s.serveJSONCodec(conn)
+	}
+}
 
-		replyType := mtype.In(2)
-		synchronous := replyType.String() != "service.RPCCallback"
-
-		if synchronous {
-			// Second arg must be a pointer.
-			if replyType.Kind() != reflect.Ptr {
-				log.Warn("method", mname, "reply type not a pointer:", replyType)
-				continue
-			}
-			// Reply type must be exported.
-			if !isExportedOrBuiltinType(replyType) {
-				log.Warn("method", mname, "reply type not exported:", replyType)
-				continue
-			}
-
-			// Method needs one out.
-			if mtype.NumOut() != 1 {
-				log.Warn("method", mname, "has wrong number of outs:", mtype.NumOut())
-				continue
-			}
-			// The return type of the method must be error.
-			if returnType := mtype.Out(0); returnType != typeOfError {
-				log.Warn("method", mname, "returns", returnType.String(), "not error")
-				continue
-			}
-		} else if mtype.NumOut() != 0 {
-			// Method needs zero outs.
-			log.Warn("method", mname, "has wrong number of outs:", mtype.NumOut())
-			continue
+func finishMethodsMapInit(methods map[string]*methodType) {
+	for name, method := range methods {
+		mtype := method.method.Type()
+		if mtype.NumIn() != 2 {
+			panic(fmt.Errorf("wrong number of inputs for method %s (%d)", name, mtype.NumIn()))
 		}
-		methods[sname+"."+mname] = &methodType{method: method, ArgType: argType, ReplyType: replyType, Synchronous: synchronous, Rcvr: rcvrv}
+		method.ArgType = mtype.In(0)
+		method.ReplyType = mtype.In(1)
+		method.Synchronous = method.ReplyType.String() != "service.RPCCallback"
 	}
 }
 
 func (s *ServerImpl) serveJSONCodec(conn io.ReadWriteCloser) {
+	clientDisconnectChan := make(chan struct{})
 	defer func() {
+		close(clientDisconnectChan)
 		if !s.config.AcceptMulti && s.config.DisconnectChan != nil {
 			close(s.config.DisconnectChan)
 		}
@@ -301,7 +250,7 @@ func (s *ServerImpl) serveJSONCodec(conn io.ReadWriteCloser) {
 				s.log.Debugf("<- %s(%T%s)", req.ServiceMethod, argv.Interface(), argvbytes)
 			}
 			replyv = reflect.New(mtype.ReplyType.Elem())
-			function := mtype.method.Func
+			function := mtype.method
 			var returnValues []reflect.Value
 			var errInter interface{}
 			func() {
@@ -310,7 +259,7 @@ func (s *ServerImpl) serveJSONCodec(conn io.ReadWriteCloser) {
 						errInter = newInternalError(ierr, 2)
 					}
 				}()
-				returnValues = function.Call([]reflect.Value{mtype.Rcvr, argv, replyv})
+				returnValues = function.Call([]reflect.Value{argv, replyv})
 				errInter = returnValues[0].Interface()
 			}()
 
@@ -333,15 +282,15 @@ func (s *ServerImpl) serveJSONCodec(conn io.ReadWriteCloser) {
 				argvbytes, _ := json.Marshal(argv.Interface())
 				s.log.Debugf("(async %d) <- %s(%T%s)", req.Seq, req.ServiceMethod, argv.Interface(), argvbytes)
 			}
-			function := mtype.method.Func
-			ctl := &RPCCallback{s, sending, codec, req, make(chan struct{})}
+			function := mtype.method
+			ctl := &RPCCallback{s, sending, codec, req, make(chan struct{}), clientDisconnectChan}
 			go func() {
 				defer func() {
 					if ierr := recover(); ierr != nil {
 						ctl.Return(nil, newInternalError(ierr, 2))
 					}
 				}()
-				function.Call([]reflect.Value{mtype.Rcvr, argv, reflect.ValueOf(ctl)})
+				function.Call([]reflect.Value{argv, reflect.ValueOf(ctl)})
 			}()
 			<-ctl.setupDone
 		}
@@ -385,9 +334,28 @@ func (cb *RPCCallback) Return(out interface{}, err error) {
 		outbytes, _ := json.Marshal(out)
 		cb.s.log.Debugf("(async %d) -> %T%s error: %q", cb.req.Seq, out, outbytes, errmsg)
 	}
+
+	if cb.hasDisconnected() {
+		return
+	}
+
 	cb.s.sendResponse(cb.sending, &cb.req, &resp, out, cb.codec, errmsg)
 }
 
+func (cb *RPCCallback) DisconnectChan() chan struct{} {
+	return cb.disconnectChan
+}
+
+func (cb *RPCCallback) hasDisconnected() bool {
+	select {
+	case <-cb.disconnectChan:
+		return true
+	default:
+	}
+
+	return false
+}
+
 func (cb *RPCCallback) SetupDoneChan() chan struct{} {
 	return cb.setupDone
 }
@@ -400,13 +368,10 @@ func (s *RPCServer) GetVersion(args api.GetVersionIn, out *api.GetVersionOut) er
 	return s.s.debugger.GetVersion(out)
 }
 
-// Changes version of the API being served.
+// SetApiVersion changes version of the API being served.
 func (s *RPCServer) SetApiVersion(args api.SetAPIVersionIn, out *api.SetAPIVersionOut) error {
-	if args.APIVersion < 2 {
-		args.APIVersion = 1
-	}
-	if args.APIVersion > 2 {
-		return fmt.Errorf("unknown API version")
+	if args.APIVersion != 2 {
+		return errors.New("unknown API version")
 	}
 	s.s.config.APIVersion = args.APIVersion
 	return nil
@@ -425,6 +390,7 @@ type internalErrorFrame struct {
 }
 
 func newInternalError(ierr interface{}, skip int) *internalError {
+	logflags.Bug.Inc()
 	r := &internalError{ierr, nil}
 	for i := skip; ; i++ {
 		pc, file, line, ok := runtime.Caller(i)
